package org.freehep.jas.services;

import java.util.List;

/**
 * This interface aims to provide a simple way for plugins to produce plots,
 * without depending on any particular plotter implementation.
 * @author tonyj
 */
public interface PlotFactory
{
   /**
    * Returns a plotter which is able to plot the given class of data.
    */
   Plotter createPlotterFor(Class dataType);
   /**
    * Returns a plotter which is able to plot all of the given classes of data.
    */
   Plotter createPlotterFor(Class[] dataType);
   /**
    * Tests if can create a plotter for the given data type
    */
   boolean canCreatePlotterFor(Class dataType);
   /**
    * Tests if can create a plotter for the given data types
    */
   boolean canCreatePlotterFor(Class[] dataType);
   /**
    * Create a page which can be used to display plots.
    * @param name The name, or null for a default name
    */
   PlotPage createPage(String name);
   /** Returns the currently selected plot page,
    * or null if no plot page is currently selected.
    */   
   PlotPage currentPage();
   /** Returns a list of all PlotPage's which are showing.
    */
   List pages();  
   /**
    * Register a Plotter adapter that can convert an Object of class "from"
    * to an object of class "to".
    * @param adapter The PlotterAdapter.
    * @param from    The class of the objects that can be converted.
    * @param to      The class to which the objects can be converted.
    *
    */
   void registerAdapter(PlotterAdapter adapter, Class from, Class to);
}